"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.breakpointNext = breakpointNext;
exports.breakpointMin = breakpointMin;
exports.breakpointMax = breakpointMax;
exports.breakpointInfix = breakpointInfix;
exports.mediaBreakpointUp = mediaBreakpointUp;
exports.mediaBreakpointDown = mediaBreakpointDown;
exports.mediaBreakpointBetween = mediaBreakpointBetween;
exports.mediaBreakpointOnly = mediaBreakpointOnly;
exports.default = exports.defaultProps = void 0;

var _unitUtils = _interopRequireDefault(require("@bootstrap-styled/utils/lib/unitUtils"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultProps = {
  '$grid-breakpoints': {
    xs: '0',
    sm: '576px',
    md: '768px',
    lg: '992px',
    xl: '1200px'
  }
};
/**
 * @private
 * @description Name of the next breakpoint, or null for the last breakpoint.
 * @param {string} name
 * @param {object} breakpoints
 * @param {array} breakpointNames
 * @returns {*}
 */
// Breakpoint viewport sizes and media queries.
//
// Breakpoints are defined as a map of (name: minimum width), order from small to large:
//
//    (xs: 0, sm: 544px, md: 768px)
//
// The map defined in the `$grid-breakpoints` global variable is used as the `$breakpoints` argument by default.
// Name of the next breakpoint, or null for the last breakpoint.
//
//    >> breakpoint-next(sm)
//    md
//    >> breakpoint-next(sm, (xs: 0, sm: 544px, md: 768px))
//    md
//    >> breakpoint-next(sm, $breakpoint-names: (xs sm md))
//    md

exports.defaultProps = defaultProps;

function breakpointNext(name) {
  var breakpoints = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-breakpoints'];
  var breakpointNames = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : Object.keys(breakpoints);
  var n = breakpointNames.indexOf(name);

  if (n !== -1 && n + 1 < breakpointNames.length) {
    return breakpointNames[n + 1];
  }

  return null;
} // Minimum breakpoint width. Null for the smallest (first) breakpoint.
//
//    >> breakpoint-min(sm, (xs: 0, sm: 544px, md: 768px))
//    544px

/**
 * @private
 * @description Minimum breakpoint width. Null for the smallest (first) breakpoint.
 * @param {string} name
 * @param {object} breakpoints
 * @returns {*}
 */


function breakpointMin(name) {
  var breakpoints = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-breakpoints'];
  var min = breakpoints[name];
  return min !== '0' ? min : null;
} // Maximum breakpoint width. Null for the largest (last) breakpoint.
// The maximum value is calculated as the minimum of the next one less 0.1.
//
//    >> breakpoint-max(sm, (xs: 0, sm: 544px, md: 768px))
//    767px

/**
 * @private
 * @description Maximum breakpoint width. Null for the largest (last) breakpoint.
 * @param {string} name
 * @param {object} breakpoints
 * @returns {*}
 */


function breakpointMax(name) {
  var breakpoints = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-breakpoints'];
  var next = breakpointNext(name, breakpoints);

  if (next) {
    var min = _unitUtils.default.rmUnit(breakpointMin(next, breakpoints), _unitUtils.default.UNIT.PX);

    return (min - 1).toString() + _unitUtils.default.UNIT.PX;
  }

  return null;
} // Returns a blank string if smallest breakpoint, otherwise returns the name with a dash infront.
// Useful for making responsive utilities.
//
//    >> breakpoint-infix(xs, (xs: 0, sm: 576px, md: 768px, lg: 992px, xl: 1200px))
//    ""  (Returns a blank string)
//    >> breakpoint-infix(sm, (xs: 0, sm: 576px, md: 768px, lg: 992px, xl: 1200px))
//    "-sm"

/**
 * @private
 * @description Returns a blank string if smallest breakpoint, otherwise returns the name with a dash infront.
 * @param {string} name
 * @param {object} breakpoints
 * @returns {*}
 */


function breakpointInfix(name) {
  var breakpoints = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-breakpoints'];
  return breakpointMin(name, breakpoints) == null ? '' : "-".concat(name);
}
/**
 * @public
 * @description Media of at least the minimum breakpoint width. No query for the smallest breakpoint. Makes the @content apply to the given breakpoint and wider.
 * @param {string} name
 * @param {object} breakpoints
 * @param {string} content
 * @returns {string}
 */


function mediaBreakpointUp(name) {
  var breakpoints = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-breakpoints'];
  var content = arguments.length > 2 ? arguments[2] : undefined;
  var min = breakpointMin(name, breakpoints);

  if (min) {
    return "\n      @media (min-width: ".concat(min, ") {\n        ").concat(content, "\n      }\n    ");
  }

  return "\n    ".concat(content, "\n  ");
}
/**
 * @public
 * @description Media of at most the maximum breakpoint width. No query for the largest breakpoint. MMakes the @content apply to the given breakpoint and narrower.
 * @param {string} name
 * @param {object} breakpoints
 * @param {string} content
 * @returns {string}
 */


function mediaBreakpointDown(name) {
  var breakpoints = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-breakpoints'];
  var content = arguments.length > 2 ? arguments[2] : undefined;
  var max = breakpointMax(name, breakpoints);

  if (max) {
    return "\n      @media (max-width: ".concat(max, ") {\n        ").concat(content, "\n      }\n    ");
  }

  return "\n    ".concat(content, "\n  ");
}
/**
 * @public
 * @description Media that spans multiple breakpoint widths. Makes the @content apply between the min and max breakpoints.
 * @param {string} lower
 * @param {string} upper
 * @param {object} breakpoints
 * @param {string} content
 * @returns {string}
 */


function mediaBreakpointBetween(lower, upper) {
  var breakpoints = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultProps['$grid-breakpoints'];
  var content = arguments.length > 3 ? arguments[3] : undefined;
  var min = breakpointMin(lower, breakpoints);
  var max = breakpointMax(upper, breakpoints);

  if (min && max) {
    return "\n      @media (min-width: ".concat(min, ") and (max-width: ").concat(max, ") {\n        ").concat(content, "\n      }\n    ");
  } else if (min) {
    return "\n      @media (min-width: ".concat(min, ") {\n        ").concat(content, "\n      }\n    ");
  } else if (max) {
    return "\n      @media (max-width: ".concat(max, ") {\n        ").concat(content, "\n      }\n    ");
  }

  return "\n    ".concat(content, "\n  ");
} // Media between the breakpoint's minimum and maximum widths.
// No minimum for the smallest breakpoint, and no maximum for the largest one.
// Makes the @content apply only to the given breakpoint, not viewports any wider or narrower.

/**
 * @public
 * @description Media between the breakpoint's minimum and maximum widths. Makes the @content apply only to the given breakpoint, not viewports any wider or narrower.
 * @param {string} name
 * @param {object} breakpoints
 * @param {string} content
 * @returns {string}
 */


function mediaBreakpointOnly(name) {
  var breakpoints = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-breakpoints'];
  var content = arguments.length > 2 ? arguments[2] : undefined;
  return mediaBreakpointBetween(name, name, breakpoints, content);
}

var _default = {
  defaultProps: defaultProps,
  up: mediaBreakpointUp,
  down: mediaBreakpointDown,
  between: mediaBreakpointBetween,
  only: mediaBreakpointOnly
};
exports.default = _default;