"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getGridColumn = getGridColumn;
exports.getColumnGridColumn = getColumnGridColumn;
exports.getMediaBreakpointUp = getMediaBreakpointUp;
exports.makeGridColumns = makeGridColumns;
exports.default = exports.defaultProps = void 0;

var _grid = require("./grid");

var _breakpoints = require("./breakpoints");

var defaultProps = {
  '$grid-gutter-widths': {
    xs: '30px',
    sm: '30px',
    md: '30px',
    lg: '30px',
    xl: '30px'
  },
  '$grid-breakpoints': {
    xs: '0',
    sm: '576px',
    md: '768px',
    lg: '992px',
    xl: '1200px'
  },
  '$grid-columns': 12,
  '$enable-grid-classes': true
}; // Framework grid generation
//
// Used only by Bootstrap to generate the correct number of grid classes given
// any value of `$grid-columns`.

exports.defaultProps = defaultProps;

function getGridColumn() {
  var gridGutterWidths = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultProps['$grid-gutter-widths'];
  var breakpoints = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-breakpoints'];
  return "\n    position: relative;\n    width: 100%;\n    min-height: 1px; /* Prevent columns from collapsing when empty */\n    ".concat((0, _grid.makeGutters)(gridGutterWidths, breakpoints), "\n  ");
}

function getColumnGridColumn() {
  var gridColumns = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultProps['$grid-columns'];
  var gridBreakpoints = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-breakpoints'];
  var gridGutterWidths = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultProps['$grid-gutter-widths'];
  var breakpoint = arguments.length > 3 ? arguments[3] : undefined;
  var columnList = [];
  var infix = (0, _breakpoints.breakpointInfix)(breakpoint, gridBreakpoints);

  for (var i = 1; i <= gridColumns; i += 1) {
    var column = "\n      &.col".concat(infix, "-").concat(i, ",\n       & .col").concat(infix, "-").concat(i, "{\n        ").concat(getGridColumn(gridGutterWidths, gridBreakpoints), "\n      }\n    ");
    columnList.push(column);
  }

  return "\n    /* Allow columns to stretch full width below their breakpoints */\n    &.col".concat(infix, ",\n     & .col").concat(infix, "{\n      ").concat(getGridColumn(gridGutterWidths, gridBreakpoints), "\n    }\n\n    ").concat(columnList.join('\n'), "\n  ");
}

function getMediaBreakpointUp() {
  var gridColumns = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultProps['$grid-columns'];
  var gridBreakpoints = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-breakpoints'];
  var breakpoint = arguments.length > 2 ? arguments[2] : undefined;
  var infix = (0, _breakpoints.breakpointInfix)(breakpoint, gridBreakpoints);
  var basic = "\n    &.col".concat(infix, ",\n     & .col").concat(infix, "{\n      flex-basis: 0;\n      flex-grow: 1;\n      max-width: 100%;\n    }\n    &.col").concat(infix, "-auto,\n     & .col").concat(infix, "-auto{\n      flex: 0 0 auto;\n      width: auto;\n    }\n  ");
  var columnList = [];

  for (var i = 1; i <= gridColumns; i += 1) {
    var column = "\n      &.col".concat(infix, "-").concat(i, ",\n       & .col").concat(infix, "-").concat(i, "{\n        ").concat((0, _grid.makeCol)(i, gridColumns), "\n      }\n    ");
    columnList.push(column);
  }

  var modifierList = ['pull', 'push'];
  var columnModifierList = [];
  modifierList.forEach(function (modifier) {
    for (var _i = 0; _i <= gridColumns; _i += 1) {
      var columnModifier = "\n        &.".concat(modifier).concat(infix, "-").concat(_i, ",\n         & .").concat(modifier).concat(infix, "-").concat(_i, "{\n          ").concat((0, _grid.makeColModifier)(modifier, _i, gridColumns), "\n        }\n      ");
      columnModifierList.push(columnModifier);
    }
  }); // `$columns - 1` because offsetting by the width of an entire row isn't possible

  var offsetColumnList = [];

  for (var _i2 = 0; _i2 <= gridColumns - 1; _i2 += 1) {
    if (infix !== 1 || _i2 !== 0) {
      // Avoid emitting useless .offset-xs-0
      var offsetColumn = "\n        &.offset".concat(infix, "-").concat(_i2, ",\n         & .offset").concat(infix, "-").concat(_i2, "{\n          ").concat((0, _grid.makeColModifier)('offset', _i2, gridColumns), "\n        }\n      ");
      offsetColumnList.push(offsetColumn);
    }
  }

  return (0, _breakpoints.mediaBreakpointUp)(breakpoint, gridBreakpoints, "\n    ".concat(basic, "\n    ").concat(columnList.join('\n'), "\n    ").concat(columnModifierList.join('\n'), "\n    ").concat(offsetColumnList.join('\n'), "\n  "));
}

function makeGridColumns() {
  var enableGridClasses = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultProps['$enable-grid-classes'];
  var gridColumns = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultProps['$grid-columns'];
  var gridGutterWidths = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultProps['$grid-gutter-widths'];
  var gridBreakpoints = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : defaultProps['$grid-breakpoints'];

  if (enableGridClasses) {
    var gridColumnList = [];
    Object.keys(gridBreakpoints).forEach(function (breakpoint) {
      var gridColumn = "\n        ".concat(getColumnGridColumn(gridColumns, gridBreakpoints, gridGutterWidths, breakpoint), "\n        ").concat(getMediaBreakpointUp(gridColumns, gridBreakpoints, breakpoint), "\n      ");
      gridColumnList.push(gridColumn);
    });
    return "\n      ".concat(gridColumnList.join('\n'), "\n    ");
  }

  return '';
}

var _default = {
  defaultProps: defaultProps,
  getGridColumn: getGridColumn,
  getColumnGridColumn: getColumnGridColumn,
  getMediaBreakpointUp: getMediaBreakpointUp,
  makeGridColumns: makeGridColumns
};
exports.default = _default;